package martin;

import martin.jbpm.ext.JBPMWorkflowService;
import martin.tasks.SynchronousTask;
import martin.tasks.Task;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.concurrent.ConcurrentLinkedQueue;

/**
 * Runner
 *
 * @author Martin Dale Lyness
 * @since 8/27/12
 */
public class Runner {

    public static void main(String[] args) throws InterruptedException {
        ApplicationContext context = new ClassPathXmlApplicationContext("applicationContext.xml");
        final JBPMWorkflowService workflowService = context.getBean(JBPMWorkflowService.class);
        final Task task = context.getBean("synchronousTask", Task.class);
        final ConcurrentLinkedQueue<Long> processIds = new ConcurrentLinkedQueue<Long>();
        MyLog.log("Queueing 10 workflow processes...");
        MyLog.flush();
        for(int i=0; i<10; i++)
           new Thread("Spawn Workflow "+i) { public void run() { try{ synchronized (processIds) { processIds.add(task.doTask(null));} } catch(Throwable t) { t.printStackTrace(); System.exit(-1); }} }.start();
        Thread.sleep(2000);

        StatefulKnowledgeSession knowledgeSession = context.getBean(StatefulKnowledgeSession.class);
        boolean allComplete = false;
        checkFromBeginning: while(!allComplete) {
            synchronized (processIds) {
                for(Long procId: processIds) {
                    ProcessInstance inst = knowledgeSession.getProcessInstance(procId);
                    Thread.sleep(1000);
                    MyLog.flush();
                    if(inst!=null) {
                        continue checkFromBeginning;
                    }
                }
            }
            allComplete = true;
        }
        MyLog.log("All Completed");
        MyLog.flush();

        System.exit(0);
    }
}
