package martin.jbpm.ext;

import martin.MyLog;
import org.drools.process.instance.WorkItem;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.drools.runtime.process.WorkflowProcessInstance;
import org.jbpm.workflow.instance.node.WorkItemNodeInstance;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;
import java.util.HashMap;

/**
 * JBPMWorkflowService
 *
 * @author Martin Dale Lyness
 * @since 8/27/12
 */
@Named
@Singleton
public class JBPMWorkflowService {
    @Resource private String WORKFLOW_NAME;
    @Inject private StatefulKnowledgeSession knowledgeSession;

    //@Transactional(value = "workflow", propagation = Propagation.REQUIRES_NEW)
    public Long completeWorkItem(Long jbpmProcessId) {
        if(jbpmProcessId==null) {
            MyLog.log("*** Starting new workflow '" + WORKFLOW_NAME + "'");
            return knowledgeSession.startProcess(WORKFLOW_NAME, new HashMap<String, Object>()).getId();
        } else {
            ProcessInstance instance = knowledgeSession.getProcessInstance(jbpmProcessId);
            WorkflowProcessInstance wfInstance = (WorkflowProcessInstance) instance;
            WorkItemNodeInstance workNodeInst = (WorkItemNodeInstance)wfInstance.getNodeInstances().iterator().next();
            WorkItem workItem = workNodeInst.getWorkItem();
            assert workItem.getProcessInstanceId() == jbpmProcessId : "Unexpected process id mis-match.";

            MyLog.log("*** Completing work item '"+workItem.getName()+"' for '"+jbpmProcessId+"'");
            knowledgeSession.getWorkItemManager().completeWorkItem(workItem.getId(), new HashMap<String, Object>());
            return jbpmProcessId;
        }
    }
}
